import inquirer from 'inquirer';
import chalk from 'chalk';
import fs from 'fs';
import { resolve } from 'path';
import ncp from 'ncp';
import mkdirp from 'mkdirp';

import { createDefaultName, changeNameInfile } from '../utils/utils';

async function promptForQuestions() {
  const questions = [
    {
      type: 'input',
      name: 'directory',
      message: 'Choose a directory name for your project:',
      validate: function(value) {
        const pass = /^[a-zA-Z0-9-_]+$/.test(value);

        if (pass) {
          return true;
        }

        return 'Please enter a valid directory name';
      }
    },
    {
      type: 'input',
      name: 'name',
      message: 'Choose a name for your component',
      default: (current) => createDefaultName(current.directory),
      validate: function(value) {
        const pass = /(\w+-)+\w+/.test(value);

        if (pass) {
          return true;
        }

        return 'Name must be snake-case and must contain at least two words';
      }
    },
    {
      type: 'input',
      name: 'description',
      message: 'Give your component a description (optional)'
    }
  ];

  console.log('');
  const options = inquirer.prompt(questions);
  return options;
}

async function copyTemplate(projectName) {
  const currentDirectory = process.cwd();
  const templateDirectory = fs.realpathSync(resolve(__dirname, `../../templates/js`));

  //   const projectDirectory = await new Promise((resolve, reject) => {
  //     const projectDir = `${currentDirectory}/../${projectName}`;
  //     // mkdirp(projectDir, (err) => {
  //     //   if (err) {
  //     //     reject('Could not create directory: ' + projectDir);
  //     //   }

  //     //   resolve(projectDir);
  //     // });

  //     mkdirp(projectDir)
  //       .then((made) => {
  //         console.log(`made directories, starting with ${made}`);
  //         console.log(projectDir);
  //         resolve(projectDir);
  //       })
  //       .catch((err) => {
  //         console.log(err);
  //         reject('Could not create directory: ' + projectDir);
  //       });
  //   });

  const projectDir = `${currentDirectory}/../${projectName}`;
  // const projectDir = `${currentDirectory}/${projectName}`;
  const projectDirectory = await mkdirp(projectDir)
    .then((made) => {
      console.log(`Directory Created: ${made}`);

      new Promise((resolve, reject) => {
        ncp.ncp(templateDirectory, projectDir, (err) => {
          if (err) {
            reject('Could not copy template files');
          }

          resolve(projectDir);
        });
      });
    })
    .catch((err) => {
      console.log(err);
      reject('Could not create directory: ' + projectDir);
    });

  return projectDir;
}

export default async function createProject() {
  const options = await promptForQuestions();
  const projectDirectory = await copyTemplate(options.directory);
  setTimeout(() => {
    writeComponentName(projectDirectory, options.name);

    const finishedMessage = `
      Your project is ready!
      To get started:
  
        cd ${options.directory}
        npm i
        npm start
  
      The project will be running at: ${chalk.magenta('http://localhost:3000')}
  
    `;

    console.log(chalk.greenBright(finishedMessage));
  }, 1000);
}

async function writeComponentName(projectDirectory, name) {
  await changeNameInfile(`${projectDirectory}/public/index.html`, new RegExp(/%component-name%/g), name);
  await changeNameInfile(`${projectDirectory}/package.json`, new RegExp(/%component-name%/g), name);
  await changeNameInfile(`${projectDirectory}/src/index.jsx`, new RegExp(/%component-name%/g), name);
}
