/* eslint-disable */
import React from 'react';
import ReactDOM from 'react-dom';
import { StyleSheetManager } from 'styled-components';
import retargetEvents from 'react-shadow-dom-retarget-events';

import App from './app';

class CustomWebComponent extends HTMLElement {
  mountPoint;
  shadowRoot;
  componentAttributes = {};
  componentProperties = {};
  styledComponentMountPoint;

  constructor() {
    super();
    this.initializeShadowRoot();
    this.initializeStyledMountPoint();
    this.initializeReactMountPoint();
  }

  connectedCallback() {
    this.renderApp();
  }

  disconnectedCallback() {
    ReactDOM.unmountComponentAtNode(this.mountPoint);
  }

  static get observedAttributes() {
    return [ 'app', 'usertype', 'token' ];
  }

  attributeChangedCallback(name, oldVal, newVal) {
    this.componentAttributes[name] = newVal;

    this.renderApp();
  }

  reactProps() {
    return { ...this.componentAttributes, ...this.componentProperties };
  }

  initializeShadowRoot() {
    this.shadowRoot = this.attachShadow({ mode: 'open' });
  }

  initializeReactMountPoint() {
    if (!this.mountPoint) {
      this.mountPoint = document.createElement('div');
      this.shadowRoot.appendChild(this.mountPoint);
      retargetEvents(this);
    }
  }

  initializeStyledMountPoint() {
    if (!this.styledComponentMountPoint) {
      this.styledComponentMountPoint = document.createElement('span');
      this.styledComponentMountPoint.id = 'spa-styled-component';
      this.shadowRoot.appendChild(this.styledComponentMountPoint);
    }
  }

  renderApp() {
    ReactDOM.render(
      <StyleSheetManager target={this.styledComponentMountPoint}>
        <App {...this.reactProps()} />
      </StyleSheetManager>,
      this.mountPoint
    );
  }
}

window.customElements.define('%component-name%', CustomWebComponent);
